package rac;

import listFW.*;
import listFW.factory.*;
import lrs.*;

/**
 * Implements a factory for restricted access containers.  These
 * restricted access containers are implemented using an LRStruct to
 * hold the data objects.
 */
public abstract class ALRSRACFactory implements IRACFactory {
  
  /**
   * Utility algo to check if the LRS is empty.
   * Returns true if the host is empty, false otherwise.
   */
  private static final IAlgo CheckEmpty = new IAlgo(){
    public Object emptyCase(LRStruct host, Object... input) {
      return true;
    }
    
    public Object nonEmptyCase(LRStruct host, Object... input) {
      return false;
    }
  };
  
  /**
   * Implements a general-purpose restricted access container using
   * an LRStruct.  How? 
   *   
   * Insertion is delegated to a strategy routine
   * which is provided to the container.  This strategy
   * varies to implement the desired kind of container, e.g., queue
   * vs. stack.
   *
   * Removal is also delegated to a strategy routine
   * which is provided to the container.  This strategy varies 
   * to implement the desired kind of container as well.
   * This routine returns the LRStruct host where the 
   * desired element is located, not the element itself.  
   * The get and peek methods of the container will remove or 
   * simply get the first of that list respectively.
   * 
   * This inner class is protected so that classes derived from its
   * factory can reuse it to create other kinds of restricted access
   * container. 
   */
  protected class LRSRAContainer implements IRAContainer {
    private IAlgo _insertStrategy;
    private IAlgo _retrieveStrategy;
    private LRStruct _lrs;
    
    /**
     * Constructor for this implementation of a RAC.
     * @param insertStrategy An IAlgo that takes one parameter and inserts it into an LRStruct at the appropriate position.  Its return value is ignored. 
     * @param retrieveStrategy An IAlgo that takes no parameters and returns the LRStruct host where the desired element is located.
     */
    public LRSRAContainer(IAlgo insertStrategy, IAlgo retrieveStrategy) {
      _insertStrategy = insertStrategy;
      _retrieveStrategy = retrieveStrategy;
      clear();
    }
    
    /**
     * Empty the container.
     */
    public void clear() {
      _lrs = new LRStruct();
    }
    
    /**
     * Return TRUE if the container is empty; otherwise, return
     * FALSE.
     */
    public boolean isEmpty() {
      return (Boolean)_lrs.execute(CheckEmpty);
    }
    
    /**
     * Return TRUE if the container is full; otherwise, return
     * FALSE. 
     *
     * This implementation can hold an arbitrary number of
     * objects.  Thus, always return false.
     */
    public boolean isFull() {
      return false;
    }
    
    /**
     * Return an immutable list of all elements in the container.
     */
    public IList elements(final IListFactory fact) {
      return (IList)_lrs.execute(new IAlgo() {               
        public Object emptyCase(LRStruct host, Object... nu) {
          return fact.makeEmptyList();
        }
        
        public Object nonEmptyCase(LRStruct host, Object... nu) {
          return fact.makeNEList(host.getFirst(),
                                 (IList)host.getRest().execute(this));
        }
      });
    }
    
    /**
     * Remove the next item from the container and return it.
     */
    public Object get() {
      // STUDENT TO MODIFY
      return _lrs.removeFront(); // This is the old code
    }
    
    /**
     * Add an item to the container.
     */
    public void put(Object input) {
      _lrs.execute(_insertStrategy, input);
    }
    
    /**
     * Return the next item in the container but do not remove it.
     */
    public Object peek() {
      // STUDENT TO MODIFY
      return _lrs.getFirst(); // This is the old code
    }
  }    
}

